==> jmb.filters-1.0.tar.gz: jmb.filters-1.0/jmb/filters/utils.py <==
# coding: utf-8


class ParseDef:
    """
    Parse a definition as in:

        title__icontains
        user__username__icontains
        user__date_joined__lte
        user__date_create__year__lt
        dates__festival__name__icontains
        title__icontains:TitleLabel

    """

    def __init__(self, filter_def, model):
        """Return an object that has parsed tokend as attributes

        :arg filter_def: eg: title__icontains (see class doc fopr more examples)
        :arg model: the model that field filtering applies to
        """
        # filter_path: title / user__name /  dates__festival__name

        filter_name, *label = filter_def.split(':')
        filter_tokens = filter_name.split('__')
        self.label = label and label[0] or None
        last_field = None
        lookup = None
        transform = None
        last_model = model

        for j, token in enumerate(filter_tokens):

            if lookup:
                transform = last_field.get_transform(token)
                continue
            if last_field:
                lookup = last_field.get_lookup(token)
                if lookup:
                    continue
                transform = last_field.get_transform(token)
                if transform:
                    continue

            last_field = last_model._meta.get_field(token)
            last_model = last_field.related_model or last_field.model
            field_path = '__'.join(filter_tokens[:j+1])

        self.filter_path = field_path
        self.lookup_type = lookup and lookup.lookup_name or 'exact'
        self.transform = transform and transform.lookup_name or None
        if self.transform:
            self.lookup_expr = f'{self.transform}__{self.lookup_type}'
        else:
            self.lookup_expr = f'{self.lookup_type}'

        ## needed just for labels
        if self.lookup_type == 'exact':
            self.filter_name = f'{self.filter_path}'
        else:
            self.filter_name = f'{self.filter_path}__{self.lookup_expr}'

    def as_dict(self):
        return {
            self.filter_path: [self.lookup_expr]
        }

    def label_as_dict(self):
        return {
            self.filter_name: self.label
        }


def parse_filters_definition(definition, model):
    """Parse a definition that has a tuple of tuples
    each inner tuple is a definition as::

      user__username__icontains

    :arg definition: definition as defined above
    :arg model: the admin model
    """
    fields_specs = {}
    labels = {}

    for row in definition:
        for filter_definition in row:
            field_spec = ParseDef(filter_definition, model)
            ## it's important to add filter lookups in case other already exists
            ## title and title__icontains amy coexist as well as date__lte date__gte
            ## the result must be {'date': ['lte', 'gte']}
            if field_spec.filter_path in fields_specs:
                fields_specs[field_spec.filter_path] += [field_spec.lookup_expr]
            else:
                fields_specs[field_spec.filter_path] = [field_spec.lookup_expr]
            labels.update(field_spec.label_as_dict())
    return fields_specs, labels

==> jmb.filters-1.2.7.tar.gz: jmb.filters-1.2.7/jmb/filters/utils.py <==
# coding: utf-8


class ParseDef:
    """
    Parse a definition as in:

        title__icontains
        user__username__icontains
        user__date_joined__lte
        user__date_create__year__lt
        dates__festival__name__icontains
        title__icontains:TitleLabel

    """

    def __init__(self, filter_def, model):
        """Return an object that has parsed tokens as attributes

        :arg filter_def: eg: title__icontains (see class doc for more examples)
        :arg model: the model that field filtering applies to
        """
        # filter_path: title / user__name /  dates__festival__name
        filter_name, *label = filter_def.split(':')
        filter_tokens = filter_name.split('__')
        self.label = label and label[0] or None
        last_field = None
        lookup = None
        transform = None
        last_model = model

        for j, token in enumerate(filter_tokens):

            if lookup:
                transform = last_field.get_transform(token)
                continue
            if last_field:
                lookup = last_field.get_lookup(token)
                if lookup:
                    continue
                try:
                    transform = last_field.get_transform(token)
                except AttributeError:
                    ## One2One has a field attribute
                    transform = last_field.field.get_transform(token)

                if transform:
                    continue

            last_field = last_model._meta.get_field(token)
            last_model = last_field.related_model or last_field.model
            field_path = '__'.join(filter_tokens[:j+1])

        self.filter_path = field_path
        self.lookup_type = lookup and lookup.lookup_name or 'exact'
        self.transform = transform and transform.lookup_name or None
        if self.transform:
            self.lookup_expr = '{s.transform}__{s.lookup_type}'.format(s=self)
        else:
            self.lookup_expr = '{s.lookup_type}'.format(s=self)

        ## needed just for labels
        if self.lookup_type == 'exact':
            self.filter_name = '{s.filter_path}'.format(s=self)
        else:
            self.filter_name = '{s.filter_path}__{s.lookup_expr}'.format(s=self)

    def as_dict(self):
        return {
            self.filter_path: [self.lookup_expr]
        }

    def label_as_dict(self):
        return {
            self.filter_name: self.label
        }


def parse_filters_definition(definition, model):
    """Parse a definition that has a tuple of tuples
    each inner tuple is a definition as::

      user__username__icontains

    :arg definition: definition as defined above
    :arg model: the admin model
    """
    fields_specs = {}
    labels = {}

    for row in definition:
        for filter_definition in row:
            field_spec = ParseDef(filter_definition, model)
            ## it's important to add filter lookups in case other already exists
            ## title and title__icontains amy coexist as well as date__lte date__gte
            ## the result must be {'date': ['lte', 'gte']}
            if field_spec.filter_path in fields_specs:
                fields_specs[field_spec.filter_path] += [field_spec.lookup_expr]
            else:
                fields_specs[field_spec.filter_path] = [field_spec.lookup_expr]
            labels.update(field_spec.label_as_dict())
    return fields_specs, labels
